#
# Copyright (c) 2012-2014 Fixpoint Inc. All rights reserved.
# ---
#
#   セットアッププログラム用共通関数定義
#
#
trap "interrupt" SIGHUP SIGINT SIGQUIT SIGABRT SIGTERM

KOMPIRA_HOME=/opt/kompira
SERVICE=/sbin/service
PGSQL_USER="postgres"
PGSQL_DB="postgres"
PG_DATADIR=/var/lib/pgsql/9.3/data
PG_BASEDIR=`dirname $PG_DATADIR` 
HA_COUNTERPART=ha-counterpart

interrupt()
{
    echo
    echo "*** $SETUP_TYPEを中断します"
    echo
    stty echo
    rm -f $TMPFILE
    exit 1
}

current_date()
{
    date --rfc-3339=seconds
}

inquire()
{
    echo  -n "$1 "

    read answer
    case $answer in
        [yY]* ) answer="y";;
        [nN]* ) answer="n";;
        *) answer="$2" ;;
    esac
}

install_repo_direct()
{
    local repo_name=$1
    local repo_file=$2
    local repo_url=$3

    if [ ! -f /etc/yum.repos.d/$repo_file ]
    then
        echo
        echo "*** $repo_name リポジトリを追加します"
        echo
        yum-config-manager --add-repo=$repo_url
        exit_if_failed "$?" "$repo_name のインストールに失敗しました"
    fi
}

install_repo_from_rpm()
{
    local repo_name=$1
    local repo_remove=$2
    local repo_url=$3

    if ! is_yum_installed $repo_name
    then
        if [ -n "$repo_remove" ]; then
            remove_if_installed $repo_remove
        fi
        echo
        echo "*** $repo_name リポジトリを追加します"
        echo
        local dir=$(dirname $repo_url)
        local pattern=$(basename $repo_url)
        echo "--- $dir から $pattern を探します"
        local url=$(find_link $dir $pattern | tail -1)
        if [ -z "$url" ]; then
            exit_if_failed 1 "$repo_name リポジトリが見つかりません"
        fi
        echo "--- $url をインストールします"
        yum -y localinstall $url
        exit_if_failed "$?" "$repo_name のインストールに失敗しました"
        return 0
    fi
    return 1
}

find_link()
{
    #
    # url で指定した HTML ページから pattern にマッチするリンク(href)を抽出する
    #
    local url=$1
    local pattern=$2
    curl -L -s "$url/" | sed -nre "s|.*href=\"($pattern)\".*|$url/\1|p"
}

is_yum_installed()
{
    for pkg
    do
        if ! yum $YUM_OPTION list installed $pkg > /dev/null 2>&1
        then
            return 1
        fi
    done
}

exit_if_failed()
{
    local status message
    status="$1"
    message="$2"
    if [ "$status" -ne 0 ]
    then
        echo
        echo "*** [ERROR] ${message}"
        echo
        exit_setup 1
    fi
}

yum_install()
{
    local DESCRIPTION="${DESCRIPTION:-$@}"
    echo
    echo "--- $DESCRIPTION をインストールします"
    echo
    yum $YUM_OPTION -y install $@
    exit_if_failed "$?" "$DESCRIPTION のインストールに失敗しました"
}

yum_install_if_not_installed()
{
    local DESCRIPTION="${DESCRIPTION:-$@}"
    if ! is_yum_installed $@
    then
        yum_install $@
    fi
}

remove_if_installed()
{
    for pkg
    do
        if is_yum_installed $pkg
        then
            echo
            echo "*** ${pkg} を一旦アンインストールします"
            echo
            yum $YUM_OPTION -y remove $pkg
        fi
    done
}

pip_uninstall_if_installed()
{
    for pkg
    do
        if pip freeze | grep $pkg
        then
            echo
            echo "*** ${pkg} を一旦アンインストールします"
            echo
            pip uninstall -y $pkg
        fi
    done
}

service_start()
{
    local svc=$1
    echo
    echo "*** ${svc} を起動します"
    echo
    $SERVICE $svc start
    exit_if_failed "$?" "${svc} の起動に失敗しました"
}

service_stop()
{
    local svc=$1

    if $SERVICE $svc status > /dev/null 2>&1
    then
        echo
        echo "*** ${svc} を停止します"
        echo
        $SERVICE $svc stop
        exit_if_failed "$?" "${svc} の停止に失敗しました"
    fi
}

service_restart()
{
    local svc=$1
    if $SERVICE $svc status > /dev/null 2>&1
    then
        echo
        echo "*** ${svc} を再起動します"
        echo
        $SERVICE $svc restart
        exit_if_failed "$?" "${svc} の再起動に失敗しました"
    else
        service_start $svc
    fi
}

check_root()
{
    if [ `whoami` != root ]
    then
        echo "*** root ユーザーで実行してください"
        exit_setup 1
    fi
}

check_os()
{
    local CENTOS="CentOS*release 6.*"
    local AMIOS="Amazon Linux AMI release*"
    local RHOS="Red Hat Enterprise Linux Server release 6.*"
    local result

    if [ -f /etc/redhat-release ]
    then
        result=`cat /etc/redhat-release`
    fi
    if [ -f /etc/system-release ]
    then
        result=`cat /etc/system-release`
    fi
    case $result in
        $CENTOS)
            echo "*** OS のバージョンを確認しました ($result)"
            SYSTEM="CENTOS" ;;
        $AMIOS)
            echo "*** OS のバージョンを確認しました ($result)"
	    POSTGRES="postgresql93"
            SYSTEM="AWS" ;;
        $RHOS)
            echo "*** OS のバージョンを確認しました ($result)"
            # サブスクリプションの確認
            if ! subscription-manager status
            then
                echo "*** システム状態が無効です"
                echo "*** 有効なサブスクリプションをアタッチして下さい"
                exit_setup 1
            fi
            SYSTEM="RHOS" ;;
        *)
            echo "*** CentOS/Red Hat 6.0 以上が必要です"
            exit_setup 1 ;;
    esac
}

pgsql_cmddir()
{
    local psql=`which psql`
    local psql_cmd=`readlink -f $psql`
    dirname $psql_cmd
}

pgsql_replica()
{
    local master_name="$1"
    local REPL_USER="repl"

    service_stop pacemaker

    rm -f $PG_BASEDIR/tmp/PGSQL.lock

    if [ -d $PG_DATADIR.old ]
    then
        rm -rf $PG_DATADIR.old
    fi
    mv -f $PG_DATADIR $PG_DATADIR.old

    (cd / && sudo -u postgres pg_basebackup -h $master_name -U $REPL_USER -D $PG_DATADIR -X stream -P)
    exit_if_failed "$?" "$master_name のDBバックアップに失敗しました"

    service_start pacemaker
}

start_setup()
{
    echo "****************************************************************"
    echo "*** Kompira-$VERSION:"
    echo "***"
    echo "*** $SETUP_TYPEを開始します"
    echo "***     [$(current_date)]"
    echo "***"
    echo "***     ----"
    $SHOW_OPTIONS
    echo "***"
    echo
}

setup_completed()
{
    echo "***"
    echo "***  $SETUP_TYPEが完了しました"
    echo "***"
}

exit_setup()
{
    stty echo
    rm -f $TMPFILE
    echo
    echo "***"
    echo "*** $SETUP_TYPEを終了します"
    echo "***     [$(current_date)]"
    echo "***"
    echo "****************************************************************"
    sleep 1
    exit $1
}
