#!/bin/bash
#
# Copyright (c) 2012-2014 Fixpoint Inc. All rights reserved.
# ---
# Kompiraクラスタ構成セットアップスクリプト
#
# * NICは以下のように割当てる::
#   eth0 --- サービス提供用インタフェース
#   eth1 --- heartbeat用インタフェース
#
# * 各パラメータは以下のとおりとする
#                             primary         secondary
#   heartbeat ipaddress       192.168.99.1    192.168.99.2
#   hostname                  kompira-server1 kompira-server2
#
# * PostgreSQLのpostgresパスワードの設定は同じにしておくこと
#
SETUP_LOG=setup_cluster.$$.log
THIS_DIR=`dirname \`readlink -f $0\``
VERSION=`python -c 'import kompira_common.version; print kompira_common.version.VERSION'`

SERVICE=/sbin/service
CHKCONFIG=/sbin/chkconfig

KOMPIRA_CLUSTER_NAME=
KOMPIRA_SERVER_PREFIX=kompira-server
KOMPIRA_SERVER_ID=1
HEARTBEAT_NETCIDR=192.168.99.0/24
CLUSTER_VIP=

PGSQL_PASSWORD=
PGSQL_OPTIONS="-U postgres"
POSTGRES="postgresql-9.3"

SECONDARY=false
SLAVE_MODE=false
SKIP_HOSTNAME=false
NO_JOBMNGR_SETUP=false
DRY_RUN=false

#
# 冗長構成に必要なパッケージ
#
PACKAGES_FOR_HA="pacemaker corosync xinetd rsync lsyncd resource-agents pcs cman"
YUM_OPTIONS_FOR_HA="--enablerepo=rhel-ha-for-rhel-6-server-rpms --enablerepo=rhel-ha-for-rhel-6-server-eus-rpms"


show_options()
{
    echo "***     - SECONDARY             = $SECONDARY"
    echo "***     - SLAVE_MODE            = $SLAVE_MODE"
    echo "***     - SKIP_HOSTNAME         = $SKIP_HOSTNAME"
    echo "***     - NO_JOBMNGR_SETUP      = $NO_JOBMNGR_SETUP"
    echo "***     - KOMPIRA_SERVER_PREFIX = $KOMPIRA_SERVER_PREFIX"
    echo "***     - KOMPIRA_SERVER_ID     = $KOMPIRA_SERVER_ID"
    echo "***     - HEARTBEAT_NETCIDR     = $HEARTBEAT_NETCIDR"
    echo "***     - CLUSTER_VIP           = $CLUSTER_VIP"
    echo "***     - DRY_RUN               = $DRY_RUN"
}
SHOW_OPTIONS=show_options
TMPFILE=".kompira.$$.tmp"
SETUP_TYPE="Kompiraクラスタのセットアップ"

. $THIS_DIR/scripts/setup_utils.sh
. $THIS_DIR/scripts/maskconv.sh

PG_CMDDIR=`pgsql_cmddir`

cluster_name=
primary_name=
secondary_name=
primary_ip=
secondary_ip=
primary_counterpart=
secondary_counterpart=
kompira_hostname=
server_ipaddr=
opposite_ipaddr=
heartbeat_baseaddr=
heartbeat_ifaddr=
cluster_vip=
cluster_netmask=

check_cidr()
{
    local cidr=$1
    echo $cidr | egrep '^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/([0-9]{1,2})' >/dev/null && ipcalc -c $cidr
}

check_ipaddr_in_netcidr()
{
    local label=$1
    local ipaddr=$2
    local netcidr=$3
    local netmask=${netcidr#*/}
    #
    # ipaddr が netcidr の範囲に含まれていること、および
    # ネットワークまたはブロードキャストアドレスになっていないことをチェックする
    #
    if [ "$(ipcalc -n $ipaddr/$netmask)" != "$(ipcalc -n $netcidr)" ]; then
        echo "ERROR: $label($ipaddr) not in $netcidr"
        return 1
    fi
    if [ "$(ipcalc -n $ipaddr/$netmask)" == "NETWORK=$ipaddr" ]; then
        echo "ERROR: $label($ipaddr) is network address"
        return 1
    fi
    if [ "$(ipcalc -b $ipaddr/$netmask)" == "BROADCAST=$ipaddr" ]; then
        echo "ERROR: $label($ipaddr) is broadcast address"
        return 1
    fi
    return 0
}

set_params()
{
    echo
    echo "*** パラメータを設定します"
    echo
    yum_install_if_not_installed bc

    cluster_name="${KOMPIRA_CLUSTER_NAME}"
    primary_name="${KOMPIRA_SERVER_PREFIX}1"
    secondary_name="${KOMPIRA_SERVER_PREFIX}2"
    heartbeat_baseaddr=${HEARTBEAT_NETCIDR%/*}
    heartbeat_ifaddr=$(int2oct $(echo "$(oct2int $heartbeat_baseaddr)+$KOMPIRA_SERVER_ID" |bc))
    cluster_vip=${CLUSTER_VIP%/*}
    cluster_netmask=${CLUSTER_VIP#*/}
    primary_ip=$(int2oct $(echo "$(oct2int $heartbeat_baseaddr)+1" |bc))
    secondary_ip=$(int2oct $(echo "$(oct2int $heartbeat_baseaddr)+2" |bc))

    if [ -z "$cluster_name" ]; then
        cluster_name="kompira-$(echo ${KOMPIRA_SERVER_PREFIX}${HEARTBEAT_NETCIDR} | sha1sum | cut -c1-6)"
    fi
    if $SECONDARY
    then
        kompira_hostname=$secondary_name
        server_ipaddr=$secondary_ip
        opposite_ipaddr=$primary_ip
        primary_counterpart=$HA_COUNTERPART
    else
        kompira_hostname=$primary_name
        server_ipaddr=$primary_ip
        opposite_ipaddr=$secondary_ip
        secondary_counterpart=$HA_COUNTERPART
    fi

    echo "*** ---"
    echo "***     - cluster_name        = $cluster_name"
    echo "***     - primary_name        = $primary_name"
    echo "***     - secondary_name      = $secondary_name"
    echo "***     - kompira_hostname    = $kompira_hostname"
    echo "***     - heartbeat_baseaddr  = $heartbeat_baseaddr"
    echo "***     - heartbeat_ifaddr    = $heartbeat_ifaddr"
    echo "***     - cluster_vip         = $cluster_vip"
    echo "***     - cluster_netmask     = $cluster_netmask"
    echo "***     - primary_ip          = $primary_ip"
    echo "***     - secondary_ip        = $secondary_ip"
    echo "***     - server_ipaddr       = $server_ipaddr"
    echo "*** ---"

    #
    # 各アドレスが HEARTBEAT_NETCIDR の範囲にあるかをチェック
    #
    echo
    echo "*** IP アドレス範囲をチェックします"
    local result=0
    check_ipaddr_in_netcidr "heartbeat_ifaddr" $heartbeat_ifaddr $HEARTBEAT_NETCIDR
    let result+=$?
    check_ipaddr_in_netcidr "primary_ip" $primary_ip $HEARTBEAT_NETCIDR
    let result+=$?
    check_ipaddr_in_netcidr "secondary_ip" $secondary_ip $HEARTBEAT_NETCIDR
    let result+=$?
    exit_if_failed $result "invalid heartbeat-netaddr ($HEARTBEAT_NETCIDR)"
}

check_environ()
{
    echo
    echo "*** クラスタセットアップ環境を確認します"
    echo

    local result
    check_os
    #
    # RHEL High Availability アドオンのチェック
    #
    if [ $SYSTEM == "RHOS" ]
    then
	yum repolist $YUM_OPTIONS_FOR_HA
	exit_if_failed "$?" "RHEL High Availability アドオンが使用できません"
    fi
    #
    # NIC2枚設定されていること
    #
    for nic in eth0 eth1
    do
        ifconfig "$nic" > /dev/null 2>&1
        exit_if_failed "$?" "$nicが見つかりません"
        echo "*** NIC $nic を確認しました"
    done

    #
    # Kompiraがインストールされていること
    #
    result=`pip show Kompira`
    exit_if_failed "$?" "Kompiraがインストールされていません"
    echo "*** Kompiraがインストール済であることを確認しました ($result)"

    #
    # Kompiraのバージョンが1.4以上であること
    #
    result=`pip show Kompira | grep "Version: 1.[^0-3].*"`
    exit_if_failed "$?" "Kompiraのバージョンが1.4以上である必要があります"
    echo "*** Kompiraのバージョンが1.4以上であることを確認しました ($result)"
}

install_cluster_packages()
{
    echo
    echo "*** Kompiraクラスタ構成に必要なパッケージをインストールします..."
    echo
    yum clean all
    if [ $SYSTEM == "RHOS" ]
    then
	YUM_OPTION=$YUM_OPTIONS_FOR_HA yum_install_if_not_installed $PACKAGES_FOR_HA
    else
	yum_install_if_not_installed $PACKAGES_FOR_HA
    fi
    # if ! is_yum_installed pm_diskd
    # then
    #   wget "http://sourceforge.jp/frs/redir.php?m=jaist&f=%2Flinux-ha%2F55565%2Fpm_diskd-1.1-1.el6.x86_64.rpm"
    #   exit_if_failed "$?" "pm_diskdパッケージの取得に失敗しました"
    #   yum -y install pm_diskd-1.1-1.el6.x86_64.rpm
    #   exit_if_failed "$?" "pm_diskdのインストールに失敗しました"
    # fi
}

setup_network()
{
    rabbitmq_stop
    echo
    echo "*** eth1の設定を行います"
    echo
    local ifcfg_eth1_file=/etc/sysconfig/network-scripts/ifcfg-eth1
    local ifcfg_eth0_file=/etc/sysconfig/network-scripts/ifcfg-eth0
    # ifcfg-eth1 が無い時はダミーで作成する
    if [ ! -f $ifcfg_eth1_file ]; then
        echo -e "DEVICE=eth1\nONBOOT=no\nBOOTPROTO=none" > $ifcfg_eth1_file
    fi
    ( sed -e "/BOOTPROTO/c BOOTPROTO=\"static\"" \
        -e "/ONBOOT/c ONBOOT=\"yes\"" \
        -e "/DHCP_HOSTNAME/d" \
        -e "/HOSTNAME/d" \
        -e "/IPADDR/d" \
        -e "/NETMASK/d" \
        -e "/GATEWAY/d" \
        -e "/NETWORK/d" \
        -e "/BROADCAST/d" \
        $ifcfg_eth1_file && echo IPADDR=$heartbeat_ifaddr && ipcalc -nmb ${HEARTBEAT_NETCIDR} ) > $TMPFILE
    exit_if_failed "$?" "*** eth1の設定に失敗しました"
    # [注意] .old を suffix にすると eth1.old というインタフェースが作られてしまう
    cp -b $TMPFILE $ifcfg_eth1_file

    if $SKIP_HOSTNAME; then return 0; fi

    echo
    echo "*** ホスト名を設定します"
    echo
    local network_file=/etc/sysconfig/network

    hostname $kompira_hostname
    if grep "HOSTNAME" $network_file > /dev/null
    then
        sed -e "/HOSTNAME/c HOSTNAME=$kompira_hostname" $network_file > $TMPFILE
    else
        (cat $network_file; echo "HOSTNAME=$kompira_hostname") > $TMPFILE
    fi
    cp -S .old -b $TMPFILE $network_file
    # [注意] .old を suffix にすると eth1.old というインタフェースが作られてしまう
    sed -i~ -e "/DHCP_HOSTNAME/c DHCP_HOSTNAME=$kompira_hostname" \
        -e "/HOSTNAME/c HOSTNAME=$kompira_hostname" \
        $ifcfg_eth0_file

    echo
    echo "*** hostsファイルの設定を行います"
    echo
    local hosts_file=/etc/hosts
    #
    # $kompira_hostname の記述があれば削除(localhost にあると cman の起動に失敗する)
    # primary_ip と secondary_ip の行は削除して追加しなおす
    #
    ( sed -r \
          -e "s/(\s+)${kompira_hostname//./\.}(\s|\$)/\1/g" \
          -e "/^\s*${primary_ip//./\.}(\s|$)/d" \
          -e "/^\s*${secondary_ip//./\.}(\s|$)/d"
      echo "$primary_ip $primary_name $primary_counterpart"
      echo "$secondary_ip $secondary_name $secondary_counterpart"
    ) < $hosts_file > $TMPFILE
    cp -S .old -b $TMPFILE $hosts_file
    HOSTNAME=$kompira_hostname

    echo
    echo "*** iptables の設定を行います"
    echo
    iptables -I INPUT -p tcp --source $opposite_ipaddr -j ACCEPT
    iptables -I INPUT -p udp --source $opposite_ipaddr -j ACCEPT
    iptables-save > $TMPFILE
    cp -S .old -b $TMPFILE /etc/sysconfig/iptables

    echo
    echo "*** networkを再起動します"
    echo
    ifdown eth1
    ifup eth1
    # $SERVICE network restart
    exit_if_failed "$?" "*** networkを再起動しました"
}

setup_apache()
{
    echo
    echo "*** Apacheのステータス情報取得のための設定を行います"
    echo
    cp -S .old -b $THIS_DIR/config/cluster/status.conf /etc/httpd/conf.d/
    (cat /etc/sysconfig/httpd; echo 'export LANG=$HTTPD_LANG'; echo 'export LC_ALL=$HTTPD_LANG') > $TMPFILE
    cp -S .old -b $TMPFILE /etc/sysconfig/httpd

    echo
    echo "*** Apacheの自動起動設定をOFFにします"
    echo
    $CHKCONFIG httpd off
    service_stop httpd
}

setup_pgsql()
{
    mkdir -p "$PG_BASEDIR/pg_archive"
    chown postgres:postgres "$PG_BASEDIR/pg_archive"

    echo
    echo "*** PostgreSQLの自動起動設定をOFFにします"
    echo
    $CHKCONFIG $POSTGRES off
    service_stop $POSTGRES

    if $SLAVE_MODE
    then
        setup_pgsql_slave
    else
        setup_pgsql_master
    fi
}

setup_pgsql_master()
{
    echo
    echo "*** PostgreSQLストリーミングレプリケーションの設定を行います"
    echo

    sed -e "s|<HEARTBEAT_NETCIDR>|${HEARTBEAT_NETCIDR}|" \
        $THIS_DIR/config/cluster/pg_hba.conf > $TMPFILE
    cp -S .old -b $TMPFILE $PG_DATADIR/pg_hba.conf
    cp -S .old -b $THIS_DIR/config/cluster/postgresql.conf $PG_DATADIR/postgresql.conf
    chown postgres:postgres $PG_DATADIR/pg_hba.conf $PG_DATADIR/postgresql.conf
    service_start $POSTGRES

    echo
    echo "*** PostgreSQLのレプリケーションユーザーを作成します"
    echo
    psql $PGSQL_OPTIONS <<EOF
create user repl password 'repl' NOSUPERUSER REPLICATION;
EOF
    service_stop $POSTGRES
}

setup_pgsql_slave()
{
    echo
    echo "*** PostgreSQLのレプリケーションを行います"
    echo
    pgsql_replica $HA_COUNTERPART
}

rabbitmq_stop()
{
    if rabbitmqctl status > /dev/null 2>&1
    then
        echo
        echo "*** RabbitMQを停止します"
        echo
        rabbitmqctl stop_app
        rabbitmqctl reset
        rabbitmqctl stop
    fi
}

setup_rabbitmq()
{
    echo
    echo "*** RabbitMQのクラスタ設定を行います"
    echo
    echo kompira > /var/lib/rabbitmq/.erlang.cookie
    sed -e "s/<KOMPIRA-PRIMARY>/$primary_name/" \
        -e "s/<KOMPIRA-SECONDARY>/$secondary_name/" \
        $THIS_DIR/config/cluster/rabbitmq.config > $TMPFILE
    cp -S .old -b $TMPFILE /etc/rabbitmq/rabbitmq.config
    service_start rabbitmq-server
    rabbitmqctl set_policy all '.*' '{"ha-mode": "all"}'
    service_stop rabbitmq-server
    # service_restart rabbitmq-server

    echo
    echo "*** RabbitMQの自動起動設定をOFFにします"
    echo
    $CHKCONFIG rabbitmq-server off
    service_stop rabbitmq-server
}

setup_cman()
{
    echo
    echo "*** cmanの設定ファイルをコピーします"
    echo
    cp -S .old -b $THIS_DIR/config/cluster/cman /etc/sysconfig/cman
    sed -e "s/<KOMPIRA-CLUSTER-NAME>/$cluster_name/" \
        -e "s/<KOMPIRA-PRIMARY-NAME>/$primary_name/" \
        -e "s/<KOMPIRA-SECONDARY-NAME>/$secondary_name/" \
        -e "s/<KOMPIRA-PRIMARY-IP/$primary_ip/" \
        -e "s/<KOMPIRA-SECONDARY-IP/$secondary_ip/" \
        $THIS_DIR/config/cluster/cluster.conf > $TMPFILE
    ccs_config_validate -l $TMPFILE
    exit_if_failed "$?" "cluster.conf が不正です"
    cp -S .old -b $TMPFILE /etc/cluster/cluster.conf
    cp -S .old -b $THIS_DIR/config/cluster/logrotate.corosync /etc/logrotate.d/corosync
}

setup_rsync()
{
    echo
    echo "*** rsyncによるディレクトリ同期を設定します"
    echo
    cp -S .old -b $THIS_DIR/config/cluster/rsyncd.conf /etc/rsyncd.conf
    sed -e "s/disable\s*=\s*yes/disable = no/" /etc/xinetd.d/rsync > $TMPFILE
    cp -S .old -b $TMPFILE /etc/xinetd.d/rsync
    service_restart xinetd
    $CHKCONFIG xinetd on

    sed -e 's/^LSYNCD_OPTIONS=" "/#LSYNCD_OPTIONS=" "/' /etc/sysconfig/lsyncd > $TMPFILE
    cp -S .old -b $TMPFILE /etc/sysconfig/lsyncd
    sed -e "s/OPPOSITE_IP/$opposite_ipaddr/" $THIS_DIR/config/cluster/lsyncd.conf > $TMPFILE
    cp -S .old -b $TMPFILE /etc/lsyncd.conf
    $CHKCONFIG lsyncd off
    service_stop lsyncd
}

setup_kompira()
{
    echo
    echo "*** Kompiraデーモンの自動起動をOFFにします"
    echo
    $CHKCONFIG kompirad off
    stop kompirad

    echo
    echo "*** Kompiraスクリプトファイルを$KOMPIRA_HOME/binにコピーします"
    echo
    cp -S .old -b -R $THIS_DIR/scripts $KOMPIRA_HOME/bin
}

setup_pacemaker()
{
    echo
    echo "*** 各種リソーススクリプトファイルを/usr/lib/ocf/resource.d/にコピーします"
    echo
    local resource_path=/usr/lib/ocf/resource.d

    if [ ! -d $resource_path/kompira ]
    then
        mkdir $resource_path/kompira
    fi
    cp -S .old -b $THIS_DIR/config/cluster/kompirad $resource_path/kompira/kompirad
    chmod +x $resource_path/kompira/kompirad
    cp -S .old -b $THIS_DIR/config/cluster/kompira_jobmngrd $resource_path/kompira/kompira_jobmngrd
    chmod +x $resource_path/kompira/kompira_jobmngrd
    #
    # デフォルトで入る rabbitmq-server スクリプトは promote/demote に
    # 対応していないため独自のスクリプトに置き換える
    #
    cp -S .old -b $THIS_DIR/config/cluster/rabbitmq-server $resource_path/rabbitmq/rabbitmq-server
    chmod +x $resource_path/rabbitmq/rabbitmq-server

    echo
    echo "*** pacemakerの自動起動設定をONにします"
    echo
    $CHKCONFIG pacemaker on
    service_restart cman
    service_restart pacemaker

    if $SLAVE_MODE; then return 0; fi

    # Current DCが決定するのを待つ
    echo -n "waiting Current DC"
    while pcs status | grep "Current DC: NONE" > /dev/null
    do
        echo -n "."
        sleep 1
    done
    echo

    setup_pcs $NO_JOBMNGR_SETUP $HA_COUNTERPART $cluster_vip $cluster_netmask $primary_name $secondary_name
}

setup_pcs()
{
    local NO_JOBMNGR_SETUP=$1
    local MASTER_IP=$2
    local CLUSTER_VIP=$3
    local CLUSTER_NETMASK=$4
    local PRIMARY_NAME=$5
    local SECONDARY_NAME=$6

    pcs resource create res_httpd ocf:heartbeat:apache \
        params configfile="/etc/httpd/conf/httpd.conf" envfiles="/etc/sysconfig/httpd" port="80" \
        op start interval="0" timeout="60s" \
        op monitor interval="5s" timeout="20s" \
        op stop interval="0" timeout="60s"
    pcs resource create res_pgsql ocf:heartbeat:pgsql \
        params pgctl="$PG_CMDDIR/pg_ctl" \
          psql="psql" \
          pgdata="$PG_DATADIR" \
          tmpdir="$PG_BASEDIR/tmp" \
          rep_mode="async" \
          repuser="repl" \
          restore_command="test -f %p && test ! -f $PG_BASEDIR/pg_archive/%f && cp %p $PG_BASEDIR/pg_archive/%f" \
          primary_conninfo_opt="keepalives_idle=60 keepalives_interval=5 keepalives_count=5" \
          node_list="$PRIMARY_NAME $SECONDARY_NAME" \
          master_ip="$MASTER_IP" \
        op start   timeout="60s" interval="0s" on-fail="restart" \
        op monitor timeout="60s" interval="4s" on-fail="restart" \
        op monitor timeout="60s" interval="3s" on-fail="restart" role="Master" \
        op promote timeout="60s" interval="0s" on-fail="restart" \
        op demote  timeout="60s" interval="0s" on-fail="stop" \
        op stop    timeout="60s" interval="0s" on-fail="block" \
        op notify  timeout="60s" interval="0s"
    pcs resource create res_rabbitmq ocf:rabbitmq:rabbitmq-server \
        params mnesia_base="/var/lib/rabbitmq/mnesia" nodename="" \
        op monitor interval="5s" timeout="20s" \
        op start interval="0" timeout="600" \
        op stop interval="0" timeout="120"
    pcs resource create res_kompirad ocf:kompira:kompirad \
        op start interval="0" timeout="60s" \
        op monitor interval="5s" timeout="20s" \
        op stop interval="0" timeout="60s"
    if ! $NO_JOBMNGR_SETUP
    then
        pcs resource create res_kompira_jobmngrd ocf:kompira:kompira_jobmngrd \
            op start interval="0" timeout="60s" \
            op monitor interval="5s" timeout="20s" \
            op stop interval="0" timeout="60s"
    fi
    pcs resource create res_vip ocf:heartbeat:IPaddr2 \
        params ip="$CLUSTER_VIP" nic="eth0" cidr_netmask="$CLUSTER_NETMASK" \
        op monitor interval="10s"
    pcs resource create res_lsyncd lsb:lsyncd \
        op start interval="0s" timeout="60s" \
        op monitor interval="5s" timeout="20s" \
        op stop interval="0s" timeout="60s"

    if $NO_JOBMNGR_SETUP
    then
        pcs resource group add webserver res_vip res_httpd res_kompirad res_lsyncd
    else
        pcs resource group add webserver res_vip res_httpd res_kompirad res_kompira_jobmngrd res_lsyncd
    fi
    pcs resource meta webserver target-role="Started"
    pcs resource master ms_pgsql res_pgsql \
        master-max=1 master-node-max=1 clone-max=2 clone-node-max=1 notify=true
    pcs resource master ms_rabbitmq res_rabbitmq \
        clone-max=2 notify=true
    pcs constraint colocation add webserver ms_pgsql INFINITY with-rsc-role=Master
    pcs constraint colocation add webserver ms_rabbitmq INFINITY with-rsc-role=Master
    pcs constraint order promote ms_rabbitmq then start webserver
    pcs property set no-quorum-policy="ignore" stonith-enabled="false"
    pcs resource defaults resource-stickness="INFINITY" migration-threshold="1"
}


usage_exit()
{
    cat <<EOF
Usage: `basename $0` [OPTIONS] <cluster-vip>/<prefix>

  --cluster-name        Specify the cluster name (up to 15 characters).
  --primary             Setup for primary server.
  --secondary           Setup for secondary server.
  --master-mode         Setup with master mode.
  --slave-mode          Setup with slave mode.
  --without-jobmanager  Setup without job manager.
  --skip-hostname-setup Skip hostname setting.
  --hostname-prefix=PREFIX_NAME
                        Specify prefix of hostname.
  --heartbeat-netaddr=NETWORK_ADDRESS
                        Specify network address for heartbeat.
  --dry-run             Check the parameters without actually installing.
  --help                Show this message.

EOF
    exit 1
}

parse_options()
{
    OPTIONS=`getopt -q -o '' -l cluster-name:,primary,secondary,master-mode,slave-mode,without-jobmanager,skip-hostname-setup,hostname-prefix:,heartbeat-netaddr:,dry-run,help -- "$@"`
    [ $? != 0 ] && usage_exit

    local slave_mode
    eval set -- "$OPTIONS"
    while true
    do
        arg="$1"
        case "$arg" in
            --cluster-name)
                KOMPIRA_CLUSTER_NAME="$2"
                shift ;;
            --primary)
                SECONDARY=false
                SLAVE_MODE=false
                KOMPIRA_SERVER_ID=1 ;;
            --secondary)
                SECONDARY=true
                SLAVE_MODE=true
                KOMPIRA_SERVER_ID=2 ;;
            --master-mode)
                slave_mode=false ;;
            --slave-mode)
                slave_mode=true ;;
            --without-jobmanager)
                NO_JOBMNGR_SETUP=true ;;
            --skip-hostname-setup)
                SKIP_HOSTNAME=true ;;
            --hostname-prefix)
                KOMPIRA_SERVER_PREFIX="$2"
                shift ;;
            --heartbeat-netaddr)
                HEARTBEAT_NETCIDR="$2"
                shift ;;
            --dry-run)
                DRY_RUN=true ;;
            --) shift; break ;; # 引数は無視
            *) usage_exit ;;
        esac
        shift
    done

    if [ -n "$slave_mode" ]
    then
        SLAVE_MODE=$slave_mode
    fi

    if [ -z "$KOMPIRA_SERVER_PREFIX" ]
    then
        echo "hostname-prefix must be non-empty string."
        echo
        usage_exit
    fi

    if ! check_cidr $HEARTBEAT_NETCIDR
    then
        echo "invalid heartbeat-netaddr."
        echo
        usage_exit
    fi

    if ! $SLAVE_MODE && ! check_cidr "$1"
    then
        echo "invalid <cluster-vip>/<prefix>."
        echo
        usage_exit
    else
        CLUSTER_VIP="$1"
    fi
}


################################################################

parse_options "$@"
exec > >(tee $SETUP_LOG) 2>&1
start_setup

set_params
check_environ
if $DRY_RUN ; then
    exit_setup 0
fi
check_root
install_cluster_packages
setup_network
setup_apache
setup_pgsql
setup_rabbitmq
setup_cman
setup_rsync
setup_kompira
setup_pacemaker

setup_completed
exit_setup 0
